//Battle v1.21 - by Paul Crespy (pmc23@sprynet.com)

//Object
//
//The object of the game is to defeat a dragon.
//
//To do this, earn points by attacking and defeating lesser monsters.
//Reach certain point (XP) totals, and you advance in level and earn a health increase.
//Also after defeating monsters, you can find better weapons, armor, healing,
//or magic items.
//
//Don't view the code if you want to explore and gauge the strengh of the various
//monsters, weapons and armor, and find out the effects of the magic items for yourself.
//Essentially the point is to decide when to attack and when to run away.
//You must also choose which weapons and armor to take (you can only have one of each).
//Some weapons and armor are magical.

//History
//
//This rather simple game was my first REX program.  I used the REX SDK
//and the REX 6000 Emulator, and used the references on www.rexextras.net.
//It was actually pretty easy.  The best way to learn was through example,
//so looking at the source code from some of the other add-ins was very
//useful.
//
//As for the game itself, the idea was from a crude program I wrote for the
//TRS-80 way back in high school.  Obviously, since I wasn't restricted to
//8K, that version had more features.  A bunch of us geeks would gather in
//the computer room after school to play it.  It's really not that interesting
//today, but I thought it would be a good test to see if I could make a
//REX add-in.  I guess it'll help pass a few minutes while waiting in an
//airport though...

//Known Bugs
//
//1.	For some reason, sometimes both you and the monster will keep missing
//	each other indefinitely.  Just Retreat if this happens.
//2.	It's possible to encounter the same exact monster twice
//	(with the same results and treasure), especially if it was dispatched
//	quickly.

//Version History
//
//1.0	Initial version.
//1.1	Reworked everything and optimized executable size.
//	Was > 8K and didn't work on some REX's.  It worked on mine though!
//1.2	Removed one of the magic items to reduce the executable size to 8K.
//1.21	Switched the position of the "Take It" and "Leave It" buttons.

#include <rex/rex.h>		
#include <stdlib.h>
#include <string.h>

//#include <rex/Database.h>

#define RTC_SEC		0x30		//register holding seconds (0-9)
//			0x31		//register holding tens of seconds (0-5)
//			0x32		//register holding minutes (0-9)
//			0x33		//register holding tens of minutes (0-5)
//			0x34		//register holding hours (0-9)

//game-tuning constants
#define CHANCE_HIT		 15	//base chance to hit out of 20
#define CHANCE_MONSTER_HIT	 15	//base chance for monster to hit out of 20
#define CHANCE_FIND_WEAPON	 25	//base % chance to find a weapon
#define CHANCE_FIND_ARMOR	 15	//base % chance to find armor
#define CHANCE_MAGIC_EQUIP	 10	//base % chance weapons/armor are magical
#define CHANCE_FIND_HEALING	 25	//base % chance to find healing
#define MONSTER_LEVEL_BONUS_EQUIP 4	//bonus monster level multiplier for finding weapons or armor
#define LUCK_BONUS_EQUIP	  2	//bonus luck multiplier for finding weapons or armor
#define LUCK_BONUS_MAGIC_EQUIP	 10	//bonus luck multiplier for weapons or armor to be magic
#define LUCK_BONUS_MAGIC	  6	//bonus luck multiplier for finding healing or misc. magic
#define TARGET_XP_AFTER_TEN	300	//XP required for each "level" after 10
// CHANCE_FIND_WEAPON + CHANCE_FIND_ARMOR should not be > 60 - 5*LUCK_BONUS_EQUIP
// CHANCE_FIND_HEALING + MAX_MAGIC should not be > 80 - 5*LUCK_BONUS_HEALING

//button constants
#define ID_BUTTON_ONE	0x51		//All buttons displayed in position one must have this value
#define ID_BUTTON_TWO	0x52		//All buttons displayed in position two must have this value
#define ID_ATTACK	0x51	
#define ID_RETREAT	0x52	
#define ID_EXIT		0x53
#define ID_BATTLE       0x51
#define ID_PLAYAGAIN    0x51
#define ID_MORE         0x51
#define ID_YES          0x52
#define ID_NO		0x51

//screen mode constants (determine which buttons to show)
#define MODE_READY      0x01
#define MODE_BATTLE     0x02
#define MODE_MORE	0x03
#define MODE_CHOICE	0x04
#define MODE_REPLAY     0x05
#define MODE_NONE       0x06

//special magic constants (index to magic array)
#define LIGHTNING	0	//Next monster that is hit takes additional lightning damage
#define FIRE		1	//Next monster that hits takes fire damage
#define LUCK		2	//Increases luck (chance of finding things)
#define ENCHANTMENT	3	//Enchants weapon, adds +1
#define PROTECTION	4	//Enchants armor, adds +1
//#define KNOWLEDGE	5	//Allows additional knowledge of monster health, and magic items

#define MAX_MAGIC	5	//Number of magic items requiring storage in the array

static char *magiclist[MAX_MAGIC] = { 
	"rune of lightning!",
	"rune of fire!",
	"luckstone!",
	"rune of enchantment!",
	"rune of protection!" };
//	"rune of knowledge!" };

//Weapons and Armor
static char *weaponlist[10] = { "knife", "dagger", "club", "war hammer", "mace", "short sword", "morning star", "battle axe", "long sword", "two-handed sword" };
static char *armorlist[7] = { "cloak", "leather", "studded leather", "scalemail", "chainmail", "splintmail", "platemail" };

//player globals
int hp;
int maxhp;
int level;
int xp;
int nextlevel;
int weapon;
int armor;
int magic[MAX_MAGIC];

//monster globals
int monsterhp;
int monsterlevel;
int monsterweapon;
int monsterarmor;
int monstermaxhp;

//misc globals
int textline;  //line 1 to 4 of scrolling text area
int mode;
int gameover;

int random(int base) {

   // Generates a number from 1 to base

   int num;

   if (base == 1) {
      num = 1;
   }
   else {
      num = abs(rand() % base) + 1;
   }
   return num;
}

initializeplayer() {

   //Reset all player globals

   int i;

   maxhp = hp = random(4) + 4;		//4-8 starting hit points
   level = 1;
   xp = 0;
   nextlevel = 1000 + TARGET_XP_AFTER_TEN;	//used for level 11+ only
   armor = 0;
   weapon = 1;
   for (i=0; i < MAX_MAGIC; i++) {
      magic[i] = 0;
   }
}

showbuttons() {

   //Display the buttons on the bottom of the screen, based on mode

   const int button1left = 4;
   const int button2left = 58;
   const int button3left = 186;
   const int buttontop = 101;
   const int buttonwidth = 50;
   const int buttonheight = 17;

   DsDisplayBlockClear(1,101,110,17);
   DsEventClear();
   switch (mode) {   
      case MODE_READY:
         DsDialogTextButton(button1left,buttontop,buttonwidth,buttonheight,ID_BATTLE,"Battle!");
         break;
      case MODE_BATTLE:
         DsDialogTextButton(button1left,buttontop,buttonwidth,buttonheight,ID_ATTACK,"Attack");
         DsDialogTextButton(button2left,buttontop,buttonwidth,buttonheight,ID_RETREAT,"Retreat");
         break;
      case MODE_MORE:
         showstatus();
         DsDialogTextButton(button1left,buttontop,buttonwidth,buttonheight,ID_MORE,"More");
         break;
      case MODE_CHOICE:
         showstatus();
         DsDialogTextButton(button1left,buttontop,buttonwidth,buttonheight,ID_NO,"Leave it");
         DsDialogTextButton(button2left,buttontop,buttonwidth,buttonheight,ID_YES,"Take it");
         break;
      case MODE_REPLAY:
         DsDialogTextButton(button1left,buttontop,buttonwidth+15,buttonheight,ID_PLAYAGAIN,"Play Again");
         break;
   }
   DsDialogTextButton(button3left,buttontop,buttonwidth,buttonheight,ID_EXIT,"Exit");
}

int getPlayerInput() {

   //Get input from the keypad or the buttons on the screen

   int flag = 1;
   int response = 0;
   MSG msg;

   while (flag) {

      DsEventMessageGet(&msg);

      switch (msg.message) {

         //case MSG_DS_CLOSE:		//Not sure when this event occurs.
            //DsAddinTerminate();

         case MSG_DS_PAINT:		//This event occurs after the low batter message clears.
            DsClearScreen();
            textline = 0;
            showdisplay();
            showstatus();
            showbuttons();
            break;

         case MSG_DS_COMMAND:		//A button on the screen was pressed; return its value.
            response = msg.bCode;
            if (response == ID_EXIT) {
               DsAddinTerminate();
            }
            flag = 0;
            break;

         case MSG_DS_KEY_DOWN:		//A key from the keypad on the right was pressed.
            response = msg.sCode;
            flag = 0;
            switch (response) {
               case KEY_TOP_C:
                  DsAddinTerminate();
                  break;
               case KEY_ENTER_C:
               case KEY_ENTER_E:
                  response = ID_BUTTON_ONE;
                  break;
               case KEY_BACK_C:
               case KEY_BACK_E:
                  response = ID_BUTTON_TWO;
                  break;
               default:
                  flag = 1;
            }
            break;
      }
   }
   return response;
}

int waitforbutton(int newmode) {

   //Display buttons and wait for the player to press a certain button.

   int choice;
   int tempmode = MODE_NONE;

   if (newmode != 0) {
      tempmode = mode;
      mode = newmode;
   }

   showbuttons();

   do {
      choice = getPlayerInput();
      }
   while (choice != ID_BUTTON_ONE && choice != ID_BUTTON_TWO);

   mode = tempmode;

   return choice;
}

showdisplay() {

   //Display top title bar

   DsDisplayBlock(0,0,240,12,1);
   DsPrintf(10,1,1,"Battle v1.21");
}

print(char text[], int i, int p, int l, int x, int y) {

   char t[20];

   strcpy(t, text);
   DsCalIntToStr(i, &t[p], l, ' ');
   DsPrintf(x,y,0,t);
}

print2(char text[], int i, int p, int l, int x, int y) {

   char t[20];
   int cpos;

   strcpy(t, text);
   cpos = strlen(text);
   if (i > 0) {
      strcpy(&t[cpos]," +X\0");
      DsCalIntToStr(i, &t[p + cpos], l, ' ');
   }
   DsPrintf(x,y,0,t);

}

showstatus() {

   //Display player status

   DsDisplayBlockClear(0,13,240,36);

   print("Level: XX", level, 7, 2, 4, 13);

   print("XP: XXXX", xp, 4, 4, 64, 13);

   if (magic[LUCK] > 0) {
      print("Luck +X", magic[LUCK], 6, 1, 124, 13);
   }

   print("HP: XXX", hp, 4, 3, 172, 13);
   print("(XXX)", maxhp, 1, 3, 210, 13);

   DsPrintf(4,25,0,"Weapon: ");
   print2(weaponlist[weapon - 1], magic[ENCHANTMENT], 2, 1, 54, 25);

   if (magic[LIGHTNING] > 0) {
      DsPrintf(184,25,0,"[Lightning]");
   }

   DsPrintf(4,37,0,"Armor: ");
   print2(armorlist[armor], magic[PROTECTION], 2, 1, 42, 37);

   if (magic[FIRE] > 0) {
      DsPrintf(196,37,0,"[Fire]");
   }

   //DsDisplayLine(0,49,240,0,0x0FF);
   DsDisplayBlock(0,11,240,39,0);
}

showtextline(char text[], int hpos) {

   //Display a line of information.  There are 4 positions.
   //When the display area fills up, it will clear after the player presses the "More" button.
   //If hpos is provided, the text prints on the same line as the previous call.

   int vpos;

   if (hpos == 0) {   
      if (++textline > 4) {
         if (textline == 5) {
            //The display is printing a 5th message.
            //(textline would be 6 if it were manually set high)
            //Pause the display before continuing
            waitforbutton(MODE_MORE);
         }
         DsDisplayBlockClear(0,52,240,48);
         textline = 1;
      }
   }
   
   vpos = 52 + ((textline - 1) * 12);
   DsPrintf(hpos+4,vpos,0,text);
}

displaymonsterstatus(int initialencounter, int textpos) {

   //Relate to the player an approximate or exact indication of the monster's health.

   int hpos = 165;

   int *weak[11] =   { 0, 1, 2, 2,  5,  8, 11, 14, 17, 23, 28 };
   int *strong[11] = { 3, 4, 5, 7, 13, 19, 25, 31, 37, 49, 62 };

   char text[4];

   /* if (magic[KNOWLEDGE] > 0 && magic[KNOWLEDGE] * 2 >= monsterlevel) {	//It takes 5 KNOWLEDGE points to know hps for all monster levels
      showtextline("It appears to have",0);
      strcpy(text,"   ");
      DsCalIntToStr(monsterhp,&text[1],2,' ');
      showtextline(text,90);
      
      showtextline("health point",108);
      if (monsterhp != 1) {
         showtextline("s",hpos);
         hpos = 170;
      }
      if (!initialencounter) {
         showtextline(" remaining.",hpos);
      }
      else {
         showtextline(".",hpos);
      }
   }
   else */ if (initialencounter) {
      if (monsterhp <= weak[monsterlevel]) {
         showtextline("It looks weak.",0);
      }
      else if (monsterhp >= strong[monsterlevel]) {
         showtextline("It looks strong.",0);
      }
   }
   else {
      if (monsterhp <= monsterlevel) {
         showtextline("It is severely wounded!",textpos);	//This message may be continued on the same line.
      }
      else if (monsterhp <= weak[monsterlevel]) {
         showtextline("It is seriously wounded.",textpos);	//This message may be continued on the same line.
      }
   }
}

encounter() {

   //Select a monster at random, and set its attributes.
   //There are 11 monsters, levels 0 to 10.

   static char *monsterlist[11] = { "a gremlin.", "a kobold.", "a goblin.", "an orc.",  "a gnoll.", "a bugbear.", "an ogre.", "a troll.", "a minotaur.", "a giant.", "a dragon!" };
   static int hitdicetype[11] = { 2, 4, 6, 8, 8, 8, 8, 8, 8, 8, 8 };	//For each hitdice, monster gets 1 to X hp.
   static int hitdice[11] =     { 1, 1, 1, 1, 2, 3, 4, 5, 6, 8,10 };	//Number of times to add 1 to X hp, with X from hitdicetype.
   static int maxdamage[11] =   { 1, 2, 4, 6, 8,10,12,14,16,18,20 };	//Max damage the monster can do.
   static int defense[11] =     { 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6 };	//Defense value makes the monster harder to hit.

   int rnd;
   int i;

   rnd = random(level + 1) - 1;  	//The player may encounter monsters of level equal or less than the player's level.
   monsterlevel = rnd;
   monsterhp = 0;
   for(i = 0; i < hitdice[rnd]; i++) {
      monstermaxhp = monsterhp = monsterhp + random(hitdicetype[rnd]);
   }
   monsterweapon = maxdamage[rnd];
   monsterarmor = defense[rnd];   

   showtextline("You have encountered ",0);
   showtextline(monsterlist[rnd],112);

   displaymonsterstatus(1,0);
}

magicdamage() {

   monsterhp = monsterhp - (random(10) + level - 1);
}

attack() {

   //Execute an attack against the monster.

   //Get a random number from 1 to 20.
   //1 always misses. 20 always hits.
   //Otherwise, check if the value is greater than the value required to hit.
   //The base value is CHANCE_HIT, which is modified by:
   //	monster armor: 	    makes it tougher to hit
   //	player level:       makes it easier to hit
   //	weapon enchantment: makes it easier to hit	

   int rnd;
   int damage;
   int hpos = 0;
   char text[3];

   rnd = random(20);
   if((rnd >= (CHANCE_HIT + monsterarmor - level - magic[ENCHANTMENT]) && rnd > 1) || rnd == 20) {
      showtextline("You hit it!",0);
      damage = random(weapon) + magic[ENCHANTMENT];	//Weapon enchantment is added to damage
      if (damage < 1) {
         damage = 1;
      }
      monsterhp = monsterhp - damage;
      if (magic[LIGHTNING] > 0  && monsterhp > 0) {
         showtextline("It is blasted by lightning!",54);
         magic[LIGHTNING]--;
         magicdamage();
      }
      else {
         hpos = 54;
      }
      if (monsterhp <= 0) {
         showtextline("You destroyed it!",hpos);
         awardxp();
         treasure();
      }
      else {
         displaymonsterstatus(0,hpos);
      }
      showstatus();
   }
   else {
      showtextline("You miss it.",0);
   }
}

counterattack() {

   //Execute an attack against the player.

   //Get a random number from 1 to 20.
   //1 always misses. 20 always hits.
   //Otherwise, check if the value is greater than the value required to hit.
   //The base value is CHANCE_MONSTER_HIT, which is modified by:
   //	player armor: 	    makes it tougher to hit
   //	monster level:      makes it easier to hit
   //	armor enchantment:  makes it tougher to hit	

   int rnd;
   int damage;

   rnd = random(20);
   if((rnd >= (CHANCE_MONSTER_HIT + armor - monsterlevel + magic[PROTECTION]) && rnd > 1) || rnd == 20) {
      showtextline("It attacks and hits!",0);
      damage = random(monsterweapon) - armor;		//Magic armor protection does not absorb damage, but base armor does
      if (damage < 1) {
         damage = 1;
      }
      hp = hp - damage;
      if (hp <= 0) {
         hp = 0;
      }
      else if (magic[FIRE] > 0) {
         showtextline("Fire engulfs it!",100);
         magic[FIRE]--;
         magicdamage();
         if (monsterhp <= 0) {
            showtextline("It is destroyed!",0);
            awardxp();
            treasure();
         }
      }
      showstatus();
   }
   else {
      showtextline("It attacks but misses.",0);
   }
}

awardxp() {

   //Award points to the player for vanquishing the monster.
   //Check if the player advanced a level.

   static int targetxp[11] = {0, 10, 30, 60, 120, 220, 360, 540, 760, 1000};	//xp required for each level

   int newhp = 0;
   int award;

   award = (monsterlevel * monsterlevel) + monstermaxhp;	//This is the xp award for slaying a monster

   xp = xp + award;
   if ((level < 10  && xp >= targetxp[level]) || (level >= 10 && xp >= nextlevel)) {
      showtextline("You feel more experienced.",0);
      level++;
      newhp = random(7) + 1;	//Gain 2-8 hp for each level, even after level 10
   }
   if (level > 10) {
      level = 10;
      nextlevel = nextlevel + TARGET_XP_AFTER_TEN;	//Target xp for next "level" (Max level is 10)
   }
   hp = hp + newhp;
   maxhp = maxhp + newhp;
}

int generatebonus() {

   //Creates armor or weapon bonus of +1 to +5

   int bonus = 0;
   int rnd;

   rnd = random(100 - (magic[LUCK] * LUCK_BONUS_MAGIC_EQUIP));	//Luck makes magic weapons/armor more common.
   if (rnd <= CHANCE_MAGIC_EQUIP) {
      rnd = random(100);
      if (rnd <= 3) {
         bonus = 5;	//+5: 3%
      }
      else if (rnd <= 9) {
         bonus = 4;	//+4: 6%
      }
      else if (rnd <= 21) {
         bonus = 3;	//+3: 12%
      }
      else if (rnd <= 45) {
         bonus = 2;	//+2: 24%
      }
      else {
         bonus = 1;	//+1: 55%
      }
   }
   return bonus;
}

treasure() {

   //Randomly award weapons, armor, and magic items to the player.

   static int weaponchance[9] = {22, 19, 16, 13, 10, 8, 6, 4, 2};	//% chance to find weapons (must total 100%)
   static int armorchance[6] =  {48, 24, 14, 8, 4, 2};			//% chance to find armor (must total 100%)

   int rnd;
   int healing = 0;
   int endurance = 0;
   int newweapon = 1;
   int newarmor = 0;
   int bonus = 0;
   char text[20];
   int cpos;
   int hpos;
   int total = 0;

   const int posafterfind = 42;

   //Base 100% is used.  Each monster level or luck point reduces this,
   //making weapons and armor more common.

   //Assuming MONSTER_LEVEL_BONUS_EQUIP is 4 and LUCK_BONUS_EQUIP is 2, and the chance to find a weapon is 25%:
   //A level 0 monster with player luck of 0 yields 25 out of 100 (25%)
   //A level 10 monster with player luck of 5 yields 25 out of 50 (50%)

   rnd = random(100 - (monsterlevel*MONSTER_LEVEL_BONUS_EQUIP) - (magic[LUCK]*LUCK_BONUS_EQUIP));
   
   if (rnd <= CHANCE_FIND_WEAPON) {

      //Weapons

      rnd = random(100);
      do {
         total = total + weaponchance[++newweapon - 2];
         }
      while (rnd > total);

      bonus = generatebonus();

      showtextline("You find a",0);
      hpos = 51;
      /* if (magic[KNOWLEDGE] > 0 && bonus > 0) {		//KNOWLEDGE allows the player to detect magic weapons.
         showtextline("magic",hpos);
         hpos = 82;
      } */
      strcpy(text,weaponlist[newweapon - 1]);
      cpos = strlen(text);
      strcpy(&text[cpos],".\0");
      showtextline(text,hpos);

      if (magic[ENCHANTMENT] > 0 || newweapon < weapon) {	//If current weapon is stronger or is magical,
         if (waitforbutton(MODE_CHOICE) == ID_YES) {		//ask if player wants to take the new weapon.
            weapon = newweapon;
            magic[ENCHANTMENT] = bonus;
         }
         else {
            bonus = 0;
         }
      }
      else {
         weapon = newweapon;
         magic[ENCHANTMENT] = bonus;
      }
      if (bonus > 0) { // && magic[KNOWLEDGE] == 0) {
         showtextline("It's magical!",59 + (cpos * 6));		//There is no need to tell the player it's magical if the player has KNOWLEDGE.
      }
   }

   else if (rnd <= CHANCE_FIND_ARMOR + CHANCE_FIND_WEAPON) {

      //Armor

      rnd = random(100);
      do {
         total = total + armorchance[++newarmor - 1];
         }
      while (rnd > total);

      bonus = generatebonus();

      showtextline("You find",0);
      hpos = 42;
      /* if (magic[KNOWLEDGE] > 0 && bonus > 0) {		//KNOWLEDGE allows the player to detect magic armor.
         showtextline("magic",hpos);
         hpos = 73;
      } */
      strcpy(text,armorlist[newarmor]);
      cpos = strlen(text);
      strcpy(&text[cpos]," armor.\0");
      showtextline(text,hpos);

      if (magic[PROTECTION] > 0 || newarmor < armor) {		//If current armor is stronger or is magical,
         if (waitforbutton(MODE_CHOICE) == ID_YES) {		//Ask if player wants to take the new armor.
            armor = newarmor;
            magic[PROTECTION] = bonus;
         }
         else {
            bonus = 0;
         }
      }
      else {
         armor = newarmor;
         magic[PROTECTION] = bonus;
      }
      if (bonus > 0) { // && magic[KNOWLEDGE] == 0) {
         showtextline("It's magical!", 82 + (cpos * 5));	//There is no need to tell the player it's magical if the player has KNOWLEDGE.
      }
   }

   //Potions, Scrolls, Runes, Miscellaneous

   //The base 100% for magic items is modified by levels and luck.
   //If the player is level 10 with a luck of 5, and defeated a level 10 monster and LUCK_BONUS_MAGIC is 6:
   //A magic item can be found 1 out of 50 times instead of 1 out of 100 times.

   rnd = random(100 - (level + monsterlevel + (magic[LUCK] * LUCK_BONUS_MAGIC)));
   if (rnd <= MAX_MAGIC) {		//base 1% for each miscellaneous magic item.
      showtextline("You find a",0);
      if (magic[rnd-1] < 5) {
         magic[rnd-1]++;
      }
      showtextline(magiclist[rnd-1],52);
   }
   else if (rnd <= MAX_MAGIC + CHANCE_FIND_HEALING) {
      rnd = random(100);
      showtextline("You find",0);
      if (rnd <= 3) {  		//3%
         showtextline("an elixir of restoration!",posafterfind);
         healing = 100;
         endurance = 3;
      }
      else if (rnd <= 9) {  	//6%
         showtextline("a potion of extra healing!",posafterfind);
         healing = random(17) + 3; //abs(rand() % 9) + 2;  //4-20
         endurance = 2;
      }
      else if (rnd <= 21) {  	//12%
         showtextline("a potion of healing!",posafterfind);
         healing = random(9) + 1; //abs(rand() % 9) + 2;  //2-10
         endurance = 1;
      }
      else if (rnd <= 45) {  	//24%
         showtextline("some healing salve.",posafterfind);
         healing = random(4); //abs(rand() % 3) + 1;  //1-4
      }
      else {  			//55%
         showtextline("some medicinal herbs.",posafterfind);
         healing = 1;
      }
      if (hp + endurance >= maxhp) {
         maxhp = hp = hp + endurance;		//Potions can add to max hp
      }
      else {
         hp = hp + healing;
         if (hp > maxhp) hp = maxhp;
      }
   }
}

playagain(char text[]) {

   showtextline(text,0);
   waitforbutton(MODE_REPLAY);
   gameover = 1;
}

seedrandom() {

   //Seed the random number generator from the clock

   unsigned int seed;
   int r[5];
   int i = 0;

   do r[i] = REGISTER_READ(RTC_SEC + i) - 96;
   while (++i < 5);
   seed = (unsigned int) r[0] + (r[2]*10) + (r[4]*100) + (r[1]*(r[3]+1)*1000);
   srand(seed);
}

main() {

   //Main routine

   int runaway = 0;

   while(1) {

      seedrandom();
      initializeplayer();
      textline = 0;
      DsClearScreen();
      showdisplay();
      showstatus();
      gameover = 0;

      while (!gameover) {
         runaway = 0;
         waitforbutton(MODE_READY);
         seedrandom();
         textline = 5;  //force the text area to clear after next user action
         encounter();

         while (hp > 0 && monsterhp > 0 && !runaway) {
            switch (waitforbutton(MODE_BATTLE)) {
               case ID_ATTACK:
                  attack();
                  break;
               case ID_RETREAT:
                  runaway = 1;
                  break;
            }
            if (monsterhp > 0 && hp > 0) {
               counterattack();
               if (!runaway && hp > 0) {
                  textline = 5;  //force the text area to clear after next user action
               }
            }
         }

         if (hp <= 0) {
            playagain("You are dead!");
         }
         else if (runaway) {
            showtextline("You got away.",0);
         }
         else if (monsterlevel == 10) {
            playagain("You win!");
         }
         textline = 5;  //force the text area to clear after next user action
      }
   }

}

